/*
 * Created on 2005. 3. 22
 */
package org.manalith.action;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.actions.DispatchAction;
import org.manalith.form.BlogForm;
import org.manalith.maingate.manager.MaingateManager;
import org.manalith.maingate.resource.MaingateOption;
import org.manalith.manager.ArticleCommentManager;
import org.manalith.manager.ArticleManager;
import org.manalith.manager.BlogBookmarkManager;
import org.manalith.manager.BlogManager;
import org.manalith.manager.FileManager;
import org.manalith.manager.TemplateManager;
import org.manalith.manager.TrackbackManager;
import org.manalith.manager.VisitorLogManager;
import org.manalith.resource.Article;
import org.manalith.resource.ArticleComment;
import org.manalith.resource.Blog;
import org.manalith.resource.User;
import org.manalith.resource.VisitorLog;
import org.manalith.util.EscapeUtil;

/**
 * @author setzer
 */
public class BlogAction extends DispatchAction {
    
    private static Logger logger = Logger.getLogger(BlogAction.class);

    private String blogOwnerId = null;
    private String category = null;
    private String author = null;
    private String articleId = null;
    private String date = null;
    private String page = null;

    //TODO : 예외처리
    public ActionForward execute(
            ActionMapping mapping,
            ActionForm form,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        blogOwnerId = request.getParameter("id");
        category = request.getParameter("category");
        author = request.getParameter("author");
        articleId = request.getParameter("articleId");
        date = request.getParameter("date");
        page = request.getParameter("page");
        
        MaingateOption option = MaingateManager.getOption();
        request.setAttribute("option",option);
        
        if (request.getParameter("method") == null) {
            return main(mapping,form,request,response);
        } else {
            return dispatchMethod(
                    mapping,
                    form,
                    request,
                    response,
                    getMethodName(mapping, form, request, response, "method"));
        }
    }

    public ActionForward main(
            ActionMapping mapping,
            ActionForm form,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        BlogManager bm = BlogManager.instance();
        Blog blog = null;

        if (category != null)
            blog = bm.getBlog(blogOwnerId, category);
        else if(author != null)
            blog = bm.getBlog(blogOwnerId, new User(author));
        else if(page != null){
            try{
                int intPage = Integer.parseInt(page);
                blog = bm.getBlog(blogOwnerId, intPage);
            }catch(NumberFormatException e){
                String msg = "페이지 번호가 잘못되었습니다.";
                throw new ServletException(msg);
            }
        }else if (articleId != null){
            try{
                int intArticleId = Integer.parseInt(articleId);
                Article article = new Article();
                article.setId(intArticleId);
                blog = bm.getBlog(blogOwnerId, article);
            }catch(NumberFormatException e){
                String msg = "게시물 번호가 잘못되었습니다.";
                throw new ServletException(msg);
            }
        }else if (date != null){
            try{
                SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
                Date d = sdf.parse(date);
                blog = bm.getBlog(blogOwnerId, d);
            }catch(ParseException e){
                String msg = "날짜의 형식이 잘못되었습니다.";
                throw new ServletException(msg);
            }
        }else
            blog = bm.getBlog(blogOwnerId,1);
        
        if(blog != null){
	        List articles = blog.getArticles();
	        Article article = null;
	        List comments = null;
	        ArticleComment comment = null;
	        
	        for(int i=0;i<articles.size();i++){
	            article = (Article)articles.get(i);
	            
	            if(article.getFormat().toString().equals("text")){
	                article.setContents(EscapeUtil.escapeHtml(article.getContents()));
	            }
	                article.setContents(EscapeUtil.convertLineFeed(article.getContents()));
	                comments = article.getComments();
	                
	                for(int y=0;y<comments.size();y++){
	                    comment = (ArticleComment) comments.get(y);
	                    comment.setContents(EscapeUtil.escapeHtml(comment.getContents()));
	                    comment.setContents(EscapeUtil.convertLineFeed(comment.getContents()));
	                    comments.set(y,comment);
	                }
	                article.setComments(comments);
	                articles.set(i,article);
	        }
	        blog.setArticles(articles);
	        

	        blog.setTemplate(
	                TemplateManager.instance().getTemplate(
		                getServlet().getServletContext().getRealPath("/"),
		                blog.getTemplate().toString()
		                )
	                );

	        VisitorLogManager.instance().addLog(new VisitorLog(blogOwnerId, request));
	        //FIXME main 메서드가 두번씩 실행되는 문제(로그가 두번 기록됨)
	        
	        blog.setCalendar(ArticleManager.instance().getArticleCalendar(blogOwnerId));
        }
        
        request.setAttribute("blog", blog);
        
        if(blog != null){
           request.setAttribute("recentArticles",ArticleManager.instance().getRecentArticles(blogOwnerId, 5));
	        request.setAttribute("recentTrackbacks",TrackbackManager.instance().getRecentTrackbacks(blogOwnerId, 5));
	        request.setAttribute("recentComments",ArticleCommentManager.instance().getRecentComments(blogOwnerId, 5));
	        request.setAttribute("bookmarkCategories", BlogBookmarkManager.instance().getCategories(blogOwnerId));
        }

        return mapping.findForward("main");
    }

    public ActionForward rss(
            ActionMapping mapping,
            ActionForm form,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        Blog blog = BlogManager.instance().getBlog(blogOwnerId);
        if(blog.getAllowRSS() == false){
            String msg = "RSS 구독이 금지된 블로그입니다.";
            throw new ServletException(msg);
        }
        
        List articles = blog.getArticles();
        Article article = null;
        List comments = null;
        ArticleComment comment = null;
        
        for(int i=0;i<articles.size();i++){
            article = (Article)articles.get(i);
            article.setContents(EscapeUtil.escapeXml(article.getContents()));
            articles.set(i,article);
        }
        blog.setArticles(articles);
        
        request.setAttribute("blog", blog);

        return mapping.findForward("rss");
    }

    //	 블로그 생성
    public ActionForward create(
            ActionMapping mapping,
            ActionForm form,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        BlogForm blogForm = (BlogForm) form;
        Blog blog = new Blog();

        blog.setTitle(blogForm.getTitle());
        blog.setDescription(blogForm.getDescription());
        blog.setPageSize(blogForm.getPageSize());
        blog.setTemplate(blogForm.getTemplate());
        blog.setOwner(blogForm.getOwner());
        blog.setUrl(blogForm.getUrl());
        blog.setAllowRSS(blogForm.getAllowRSS());

        BlogManager.instance().createBlog(blog);
        
        
        if(blogForm.getTitleImage() != null){
            FileManager.instance().setBlogTitleImage(
                   blogForm.getOwner(),
    	            blogForm.getTitleImage(),
    	            getServlet().getServletContext().getRealPath("/"));
            }

            if(blogForm.getBackgroundImage() != null){
            FileManager.instance().setBlogBackgroundImage(
                   blogForm.getOwner(),
    	            blogForm.getBackgroundImage(),
    	            getServlet().getServletContext().getRealPath("/"));
            }

        return new ActionForward("/blog.do?id=" + blogOwnerId, true);
    }
    
    public ActionForward update(
            ActionMapping mapping,
            ActionForm form,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        BlogForm blogForm = (BlogForm) form;
        Blog blog = new Blog();

        blog.setTitle(blogForm.getTitle());
        blog.setDescription(blogForm.getDescription());
        blog.setPageSize(blogForm.getPageSize());
        blog.setTemplate(blogForm.getTemplate());
        blog.setOwner(blogForm.getOwner());
        blog.setUrl(blogForm.getUrl());
        blog.setAllowRSS(blogForm.getAllowRSS());

        BlogManager.instance().updateBlog(blog);
        
        if(blogForm.getTitleImage() != null && !blogForm.getTitleImage().getFileName().equals("")){
            FileManager.instance().setBlogTitleImage(
                   blogForm.getOwner(),
    	            blogForm.getTitleImage(),
    	            getServlet().getServletContext().getRealPath("/"));
            }

            if(blogForm.getBackgroundImage() != null && !blogForm.getBackgroundImage().getFileName().equals("")){
            FileManager.instance().setBlogBackgroundImage(
                   blogForm.getOwner(),
    	            blogForm.getBackgroundImage(),
    	            getServlet().getServletContext().getRealPath("/"));
            }

        return new ActionForward("/blog.do?id=" + blogOwnerId, true);
    }
    
    
}
