/*
 * Created on 2005. 4. 6
 */
package org.manalith.manager;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;


import org.apache.log4j.Logger;
import org.manalith.db.ConnectionFactory;
import org.manalith.resource.BlogBookmark;


/**
 * @author setzer
 */
public class BlogBookmarkManager {
    private Connection conn;
    private static BlogBookmarkManager manager = null;
    private static Logger logger = Logger.getLogger(BlogBookmarkManager.class);
    
    private BlogBookmarkManager(){
        try {
            conn = ConnectionFactory.getConnection();
        } catch (SQLException ex) {
            logger.error("SQLException: " + ex.getMessage()); 
            logger.error("SQLState: " + ex.getSQLState()); 
            logger.error("VendorError: " + ex.getErrorCode()); 
        }
    }
    
    public static BlogBookmarkManager instance(){
        if(manager == null){
            manager = new BlogBookmarkManager();
        }
        
        return manager;
    }
    
    public void createBookmark(BlogBookmark b){
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        
        StringBuffer sb = new StringBuffer();
        sb.append("INSERT INTO manalith_blog_bookmark(blogOwnerId,category,title,url,rssUrl,imageUrl,description) ");
        sb.append("VALUES(?,?,?,?,?,?,?)");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            
            pstmt.setString(1,b.getBlogOwnerId());
            pstmt.setString(2,b.getCategory());
            pstmt.setString(3,b.getTitle());
            pstmt.setString(4,b.getUrl());
            pstmt.setString(5,b.getRssUrl());
            pstmt.setString(6,b.getImageUrl());
            pstmt.setString(7,b.getDescription());
            
            pstmt.executeUpdate();
        }catch(SQLException e){
            logger.error(e.toString());
        }catch(ClassCastException e){
            logger.error(e.toString());
        }finally {
            if (rs != null) {
                try {
                  rs.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                rs = null; 
            } 
            if (pstmt != null) {
                try {
                    pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
    }
    
    public void updateBookmark(BlogBookmark b){
        PreparedStatement pstmt = null;
        
        StringBuffer sb = new StringBuffer();
        sb.append("UPDATE manalith_blog_bookmark ");
        sb.append("SET category=?, title=?, url=?, rssUrl=?, imageUrl=?, description=? ");
        sb.append("WHERE id=? ");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            
            pstmt.setString(1,b.getCategory());
            pstmt.setString(2,b.getTitle());
            pstmt.setString(3,b.getUrl());
            pstmt.setString(4,b.getRssUrl());
            pstmt.setString(5,b.getImageUrl());
            pstmt.setString(6,b.getDescription());
            pstmt.setInt(7,b.getId());
            
            pstmt.executeUpdate();
        }catch(SQLException e){
            logger.error(e.toString());
        }finally {
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
    }
    
    public void destroyBookmark(int id){
        PreparedStatement pstmt = null;
        
        StringBuffer sb = new StringBuffer();
        sb.append("DELETE FROM manalith_blog_bookmark ");
        sb.append("WHERE id=? ");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            
            pstmt.setInt(1,id);
            
            pstmt.executeUpdate();
        }catch(SQLException e){
            logger.error(e.toString());
        }finally {
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
    }
    
    public List getBookmarks(String userId){
        List bookmarks = new ArrayList();
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        BlogBookmark b = null;
        
        StringBuffer sb = new StringBuffer();
        sb.append("SELECT id,category,title,url,rssUrl,imageUrl,description ");
        sb.append("FROM manalith_blog_bookmark ");
        sb.append("WHERE blogOwnerId=? ");
        sb.append("ORDER BY category ASC ");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            
            pstmt.setString(1,userId);
            
            rs = pstmt.executeQuery();
            
            while(rs.next()){
                b = new BlogBookmark();
                b.setId(rs.getInt("id"));
                b.setBlogOwnerId(userId);
                b.setCategory(rs.getString("category"));
                b.setTitle(rs.getString("title"));
                b.setUrl(rs.getString("url"));
                b.setRssUrl(rs.getString("rssUrl"));
                b.setImageUrl(rs.getString("imageUrl"));
                b.setDescription(rs.getString("description"));
               
                bookmarks.add(b);
            }
        }catch(SQLException e){
            logger.error(e.toString());
        }finally {
            if (rs != null) {
                try {
                    rs.close(); 
                } catch (SQLException e) {
                    logger.error(e.toString());
                }
                rs = null; 
            }
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
        
        return bookmarks;
    }
    
    public List getCategories(String userId){
        List categories = new ArrayList();
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        
        StringBuffer sb = new StringBuffer();
        sb.append("SELECT category ");
        sb.append("FROM manalith_blog_bookmark ");
        sb.append("WHERE blogOwnerId=? ");
        sb.append("GROUP BY category");

        try{
            pstmt = conn.prepareStatement(sb.toString());
            
            pstmt.setString(1,userId);
            
            rs = pstmt.executeQuery();
            
            while(rs.next()){
                categories.add(rs.getString("category"));
            }
        }catch(SQLException e){
            logger.error(e.toString());
        }finally {
            if (rs != null) {
                try {
                    rs.close(); 
                } catch (SQLException e) {
                    logger.error(e.toString());
                }
                rs = null; 
            }
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
        
        return categories;
    }
}
